//
//          CMVSource 1.0.0 for AviSynth 2.5.x
//
//  Loads Dwarf Fortress CMV and CCMV movies. Latest release hosted at
//  http://www.gyroshot.com/cmvsource.htm
//
//          Copyright 2011 Robert Martens  robert.martens@gmail.com
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.



#include <fstream>  // ifstream
#include <ios>      // binary, in
#include <sstream>  // stringstream (returned by GetTokenValue())
#include <vector>
#include <Windows.h>

#include "avisynth.h"
#include "CmvSource.h"
#include "DfUtils.h"

#define BOOST_AUTO_TEST_MAIN
#include <boost/test/unit_test.hpp>



struct ScriptEnv {
  IScriptEnvironment* env;
  ScriptEnv();
  ~ScriptEnv() { if (env) delete env; }
};



ScriptEnv::ScriptEnv() : env(CreateScriptEnvironment(AVISYNTH_INTERFACE_VERSION))
{

  BOOST_REQUIRE( env );  
  BOOST_REQUIRE( env->FunctionExists("CMVSource") );

}



// I gave up trying to accomplish these by way of linking to CMVSource.dll (I
// couldn't work out the "couldn't find entry point" error) and just added the
// headers and source to the test project instead. I don't think it's an issue
// for tests like this, since the point is to test the individual functions;
// the combination of same can be handled by the other set of tests down below.
BOOST_FIXTURE_TEST_SUITE( white_box, ScriptEnv )

  BOOST_AUTO_TEST_CASE( DfUtils_LoadColors_FileDoesExist_ValidContent ) {

    std::vector<int> dfColors;

    try {

      dfColors = LoadColors("C:/Dwarf Fortress/31 25/data/init/colors.txt", env);

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

    int expectArr[16] = { 0, 128, 32768, 32896, 8388608, 8388736,
                          8421376, 12632256, 8421504, 255, 65280, 65535,
                          16711680, 16711935, 16776960, 16777215 };

    std::vector<int> expected;
    for (int i = 0; i < 16; ++i)
      expected.push_back(expectArr[i]);

    // Normally I'd use BOOST_CHECK_EQUAL, since in the case of a failure it
    // actually shows you the values of the items being compared, but it didn't
    // work here. Doesn't play nice with STL containers, I suppose.
    BOOST_CHECK( dfColors == expected );

  }

  BOOST_AUTO_TEST_CASE( DfUtils_LoadColors_FileDoesExist_ValidContent_ExtraText ) {

    std::vector<int> dfColors;

    try {

      dfColors = LoadColors("C:/Development/CMVSource/Testing/betweenthelines-colors.txt", env);

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

    int expectArr[16] = { 0, 128, 32768, 32896, 8388608, 8388736,
                          8421376, 12632256, 8421504, 255, 65280, 65535,
                          16711680, 16711935, 16776960, 16777215 };

    std::vector<int> expected;
    for (int i = 0; i < 16; ++i)
      expected.push_back(expectArr[i]);

    BOOST_CHECK( dfColors == expected );

  }

  BOOST_AUTO_TEST_CASE( DfUtils_LoadColors_FileDoesExist_InvalidContent ) {

    std::string result;

    try {

      std::vector<int> dfColors = LoadColors("C:/Development/CMVSource/Testing/invalidcolordata.txt", env);

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK_EQUAL( result, "CMVSource: Invalid color data in 'C:/Development/CMVSource/Testing/invalidcolordata.txt'" );

  }  

  BOOST_AUTO_TEST_CASE( Errors_DfUtils_LoadColors_FileDoesNotExist ) {

    std::string result = "";

    try {

      std::vector<int> dfColors = LoadColors("C:/No/Such/Dir/nosuchcolors.txt", env);

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK_EQUAL( result, "CMVSource: Couldn't open color file 'C:/No/Such/Dir/nosuchcolors.txt'" );

  }
  
  BOOST_AUTO_TEST_CASE( DfUtils_GetTokenValue_String_TokenDoesNotHaveColon ) {

    std::ifstream init("C:/Dwarf Fortress/31 25/data/init/init.txt", std::ios::binary | std::ios::in);
    BOOST_CHECK( init );

    std::string initBuf;
    while (init.good())
      initBuf.push_back(static_cast<char>(init.get()));
    init.close();
      
    std::string fullfont = GetTokenValue("FULLFONT", initBuf).str();
    BOOST_CHECK_EQUAL( fullfont, "curses_800x600.png");

  }

  BOOST_AUTO_TEST_CASE( DfUtils_GetTokenValue_String_TokenDoesHaveColon ) {

    std::ifstream init("C:/Dwarf Fortress/31 25/data/init/init.txt", std::ios::binary | std::ios::in);
    BOOST_CHECK( init );

    std::string initBuf;
    while (init.good())
      initBuf.push_back(static_cast<char>(init.get()));
    init.close();
      
    std::string fullfont = GetTokenValue("FULLFONT:", initBuf).str();
    BOOST_CHECK_EQUAL( fullfont, "curses_800x600.png");

  }

BOOST_AUTO_TEST_SUITE_END() // white_box fixture



// These tests are bound to be less than perfect, since despite knowing better
// I wrote the bulk of CMVSource before creating any of them. They should still
// help detect major issues, but next time I'm trying TDD from the start.
BOOST_FIXTURE_TEST_SUITE( black_box, ScriptEnv )

  BOOST_AUTO_TEST_CASE( Params_File_CmvFilenameIsAccepted_LowerCaseExtension ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25" };

    try {

      env->Invoke("CMVSource", AVSValue(args, 2));

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }

  BOOST_AUTO_TEST_CASE( Params_File_CmvFilenameIsAccepted_UpperCaseExtension ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress_uppercase.CMV", "C:/Dwarf Fortress/31 25" };

    try {

      env->Invoke("CMVSource", AVSValue(args, 2));

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }

  BOOST_AUTO_TEST_CASE( Params_File_CmvFilenameIsAccepted_MixedCaseExtension ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress_mixedcase.CmV", "C:/Dwarf Fortress/31 25" };

    try {

      env->Invoke("CMVSource", AVSValue(args, 2));

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }

  BOOST_AUTO_TEST_CASE( Params_File_CmvFilenameIsAccepted_MultipleDots ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.and.more.dots.cmv", "C:/Dwarf Fortress/31 25" };

    try {

      env->Invoke("CMVSource", AVSValue(args, 2));

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }

  BOOST_AUTO_TEST_CASE( Params_File_CcmvFilenameIsAccepted_LowerCaseExtension ) {

    // http://mkv25.net/dfma/movie-2362-superdefenses
    // This movie was one of the most recent uploaded to the DFMA when I started
    // working on these tests, so I grabbed a copy. Get it at that link, or use
    // one of your own from somewhere else.
    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/frogwarrior-SUPER DEFENSES.ccmv", "C:/Dwarf Fortress/31 25" };

    try {

      env->Invoke("CMVSource", AVSValue(args, 2));

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }

  BOOST_AUTO_TEST_CASE( Params_File_CcmvFilenameIsAccepted_UpperCaseExtension ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/frogwarrior-SUPER DEFENSES_uppercase.CCMV", "C:/Dwarf Fortress/31 25" };

    try {

      env->Invoke("CMVSource", AVSValue(args, 2));

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }

  BOOST_AUTO_TEST_CASE( Params_File_CcmvFilenameIsAccepted_MixedCaseExtension ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/frogwarrior-SUPER DEFENSES_mixedcase.CcMv", "C:/Dwarf Fortress/31 25" };

    try {

      env->Invoke("CMVSource", AVSValue(args, 2));

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }

  BOOST_AUTO_TEST_CASE( Params_File_CcmvFilenameIsAccepted_MultipleDots ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/frogwarrior-SUPER DEFENSES.and.more.dots.ccmv", "C:/Dwarf Fortress/31 25" };

    try {

      env->Invoke("CMVSource", AVSValue(args, 2));

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }
  
  BOOST_AUTO_TEST_CASE( Params_File_CmvIsLoaded_Prerelease_1000Frames ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/Prerelease/bigd.cmv", "C:/Dwarf Fortress/31 25" };
    PClip result;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 2)).AsClip();

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

    BOOST_CHECK_EQUAL( result->GetVideoInfo().num_frames, 1000 );
    BOOST_CHECK_EQUAL( result->GetVideoInfo().width, 800 );

  }
  
  BOOST_AUTO_TEST_CASE( Params_File_CmvIsLoaded_Prerelease_64Frames ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/Prerelease/digcrazy.cmv", "C:/Dwarf Fortress/31 25" };
    PClip result;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 2)).AsClip();

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

    BOOST_CHECK_EQUAL( result->GetVideoInfo().num_frames, 64 );
    BOOST_CHECK_EQUAL( result->GetVideoInfo().width, 800 );

  }

  BOOST_AUTO_TEST_CASE( Params_File_CmvIsLoaded_v10000 ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/Prerelease/2005_01_huntbug.cmv", "C:/Dwarf Fortress/31 25" };
    PClip result;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 2)).AsClip();

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

    BOOST_CHECK_EQUAL( result->GetVideoInfo().num_frames, 75794 );
    BOOST_CHECK_EQUAL( result->GetVideoInfo().width, 800 );

  }

  BOOST_AUTO_TEST_CASE( Params_File_CmvIsLoaded_v10001 ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25" };
    PClip result;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 2)).AsClip();

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

    BOOST_CHECK_EQUAL( result->GetVideoInfo().num_frames, 200 );
    BOOST_CHECK_EQUAL( result->GetVideoInfo().width, 800 );

  }

  BOOST_AUTO_TEST_CASE( Params_File_CmvIsLoaded_v10005 ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/lohi.cmv", "C:/Dwarf Fortress/31 25" };
    PClip result;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 2)).AsClip();

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

    BOOST_CHECK_EQUAL( result->GetVideoInfo().num_frames, 2 );
    BOOST_CHECK_EQUAL( result->GetVideoInfo().width, 800 );

  }

  BOOST_AUTO_TEST_CASE( Params_File_CcmvIsLoaded_v10001 ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/frogwarrior-SUPER DEFENSES.ccmv", "C:/Dwarf Fortress/31 25" };
    PClip result;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 2)).AsClip();

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

    BOOST_CHECK_EQUAL( result->GetVideoInfo().num_frames, 12995 );
    BOOST_CHECK_EQUAL( result->GetVideoInfo().width, 1000 );

  }

  BOOST_AUTO_TEST_CASE( Params_Tileset_IsString_PNG ) {

    AVSValue args[3] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25", "C:/Dwarf Fortress/31 25/data/art/curses_640x300.png" };

    try {

      env->Invoke("CMVSource", AVSValue(args, 3));

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }

  BOOST_AUTO_TEST_CASE( Params_Tileset_IsString_BMP ) {

    AVSValue args[3] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25", "C:/Dwarf Fortress/31 25/data/art/curses_800x600.bmp" };

    try {

      env->Invoke("CMVSource", AVSValue(args, 3));

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }

  BOOST_AUTO_TEST_CASE( Params_Tileset_IsClip_RGB32 ) {

    PClip tileClip = env->Invoke("ImageSource", AVSValue("C:/Dwarf Fortress/31 25/data/art/curses_640x300.png")).AsClip();
    tileClip = env->Invoke("ConvertToRGB32", AVSValue(tileClip)).AsClip();
    AVSValue args[3] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25", tileClip };
    PClip result = NULL;

    try {

      env->Invoke("CMVSource", AVSValue(args, 3));

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }

  BOOST_AUTO_TEST_CASE( Params_Tileset_IsClip_RGB24 ) {

    PClip tileClip = env->Invoke("ImageSource", AVSValue("C:/Dwarf Fortress/31 25/data/art/curses_640x300.png")).AsClip();
    tileClip = env->Invoke("ConvertToRGB24", AVSValue(tileClip)).AsClip();
    AVSValue args[3] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25", tileClip };
    PClip result = NULL;

    try {

      env->Invoke("CMVSource", AVSValue(args, 3));

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }

  BOOST_AUTO_TEST_CASE( Params_Colors_FileDoesExist_ValidContent_StandardFormat ) {

    AVSValue args[3] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25", "C:/Dwarf Fortress/DF_Phoebus_31_25v23/data/init/colors.txt" };
    const char* names[3] = { NULL, "DFdir", "colors" };

    try {

      env->Invoke("CMVSource", AVSValue(args, 3), names);

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }

  BOOST_AUTO_TEST_CASE( Params_Colors_FileDoesExist_ValidContent_ExtraText ) {

    AVSValue args[3] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25", "C:/Development/CMVSource/Testing/betweenthelines-colors.txt" };
    const char* names[3] = { NULL, "DFdir", "colors" };

    try {

      env->Invoke("CMVSource", AVSValue(args, 3), names);

    } catch (AvisynthError& err) {

      BOOST_WARN( err.msg );

    }

  }
    
  BOOST_AUTO_TEST_CASE( Errors_File_FilenameIsNotCmvOrCcmv ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/notacmv.abc", "C:/Dwarf Fortress/31 25" };
    AVSValue result = NULL;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 2)).AsClip();

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK( result.IsClip() == false );
    BOOST_CHECK_EQUAL( result.AsString(), "CMVSource: Only CMV and CCMV files supported!" );

  }

  BOOST_AUTO_TEST_CASE( Errors_File_FilenameWithMultipleDotsIsNotCmvOrCcmv ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/this.is.still.notacmv.abc", "C:/Dwarf Fortress/31 25" };
    AVSValue result = NULL;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 2)).AsClip();

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK( result.IsClip() == false );
    BOOST_CHECK_EQUAL( result.AsString(), "CMVSource: Only CMV and CCMV files supported!" );

  }

  BOOST_AUTO_TEST_CASE( Errors_File_CcmvFromCmvEditorUnsupported ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/savedfromcmveditor.ccmv", "C:/Dwarf Fortress/31 25" };
    AVSValue result = NULL;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 2)).AsClip();

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK( result.IsClip() == false );
    BOOST_CHECK_EQUAL( result.AsString(), "CMVSource: CCMVs from the CMV Editor are currently unsupported! See CMVSource.txt for details." );

  }

  BOOST_AUTO_TEST_CASE( Errors_File_BrokenFileThrowsInflateNeg3 ) {

    // This was just the standard bay12games movie included with DF, with a
    // few random bytes of the zlib data changed in a hex editor.
    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/BROKEN-THROWINFLATENEG3-bay12games.cmv", "C:/Dwarf Fortress/31 25" };
    AVSValue result = NULL;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 2)).AsClip();

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK( result.IsClip() == false );
    BOOST_CHECK_EQUAL( result.AsString(), "CMVSource: Couldn't decompress frames! inflate() error -3" );

  }

  BOOST_AUTO_TEST_CASE( Errors_DfDir_CustomDirDoesNotExist ) {

    AVSValue args[2] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Directory/Misplaced/By/Pozzo/The/Drunk" };
    AVSValue result = NULL;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 2)).AsClip();

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK( result.IsClip() == false );
    BOOST_CHECK_EQUAL( result.AsString(), "CMVSource: Invalid 'DFdir'!" );

  }

  BOOST_AUTO_TEST_CASE( Errors_Tileset_IsString_FileDoesNotExist ) {

    AVSValue args[3] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25", "C:/No/Such/Dir/nosuchtileset.png"};
    AVSValue result = NULL;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 3)).AsClip();

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK( result.IsClip() == false );
    BOOST_CHECK_EQUAL( result.AsString(), "CMVSource internal error:\nImageReader: error 'Could not open file' in DevIL library\n reading file C:/No/Such/Dir/nosuchtileset.png" );

  }

  BOOST_AUTO_TEST_CASE( Errors_Tileset_IsClip_YUY2 ) {

    PClip tileClip = env->Invoke("ImageSource", AVSValue("C:/Dwarf Fortress/31 25/data/art/curses_640x300.png")).AsClip();
    tileClip = env->Invoke("ConvertToYUY2", AVSValue(tileClip)).AsClip();
    AVSValue args[3] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25", tileClip };
    AVSValue result = NULL;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 3)).AsClip();

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK_EQUAL( result.AsString(), "CMVSource: 'tileset' clips must be RGB!" );

  }

  BOOST_AUTO_TEST_CASE( Errors_Tileset_IsClip_YV12 ) {

    PClip tileClip = env->Invoke("ImageSource", AVSValue("C:/Dwarf Fortress/31 25/data/art/curses_640x300.png")).AsClip();
    tileClip = env->Invoke("ConvertToYV12", AVSValue(tileClip)).AsClip();
    AVSValue args[3] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25", tileClip };
    AVSValue result = NULL;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 3)).AsClip();

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK_EQUAL( result.AsString(), "CMVSource: 'tileset' clips must be RGB!" );

  }

  BOOST_AUTO_TEST_CASE( Errors_Tileset_IsNotStringOrClip ) {

    bool tileClip = true;
    AVSValue args[3] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25", tileClip };
    const char* names[3] = { NULL, "DFdir", "tileset" };
    AVSValue result = NULL;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 3), names);

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK_EQUAL( result.AsString(), "Script error: the named argument \"tileset\" to CMVSource had the wrong type" );

  }

  BOOST_AUTO_TEST_CASE( Errors_Colors_FileDoesNotExist ) {
    
    AVSValue args[3] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25", "C:/No/Such/Dir/nosuchcolors.txt" };
    const char* names[3] = { NULL, "DFdir", "colors" };
    AVSValue result = NULL;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 3), names);

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK_EQUAL( result.AsString(), "CMVSource: Couldn't open color file 'C:/No/Such/Dir/nosuchcolors.txt'" );

  }
    
  BOOST_AUTO_TEST_CASE( Errors_Colors_FileDoesExist_InvalidContent ) {

    AVSValue args[3] = { "C:/Development/CMVSource/Testing/clips/dwarf_fortress.cmv", "C:/Dwarf Fortress/31 25", "C:/Dwarf Fortress/31 25/data/init/init.txt" };
    const char* names[3] = { NULL, "DFdir", "colors" };
    AVSValue result = NULL;

    try {

      result = env->Invoke("CMVSource", AVSValue(args, 3), names);

    } catch (AvisynthError& err) {

      result = err.msg;

    }

    BOOST_CHECK_EQUAL( result.AsString(), "CMVSource: Invalid color data in 'C:/Dwarf Fortress/31 25/data/init/init.txt'" );

  }

BOOST_AUTO_TEST_SUITE_END() // black_box fixture