//
//          CMVSource 1.0.0 for AviSynth 2.5.x
//
//  Loads Dwarf Fortress CMV and CCMV movies. Latest release hosted at
//  http://www.gyroshot.com/cmvsource.htm
//
//          Copyright 2011 Robert Martens  robert.martens@gmail.com
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software Foundation,
//  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.



#include "CmvSource.h"

#include <algorithm>  // transform
#include <cctype>     // tolower
#include <fstream>    // ifstream
#include <ios>        // binary, in, end, beg
#include <sstream>    // stringstream (returned by GetTokenValue())
#include <string>
#include <vector>
#include <Windows.h>

#include "avisynth.h"
#include "DfUtils.h"



AVSValue __cdecl Create_CmvSource(AVSValue args, void* user_data, IScriptEnvironment* env)
{

  std::string filename = args[0].AsString();

  // Properly validating the file contents to ensure I can read the data is
  // something I haven't yet tackled, so this brute force extension business
  // seemed like the best solution in the meantime.
  int extPos = filename.rfind('.') + 1;
  std::string ext = filename.substr(extPos, (filename.length() - extPos));
  std::transform(ext.begin(), ext.end(), ext.begin(), std::tolower);
  
  if (ext != "cmv" && ext != "ccmv")
    env->ThrowError("CMVSource: Only CMV and CCMV files supported!");

  std::ifstream cmvStrm(filename, std::ios::binary | std::ios::in);

  if (!cmvStrm)
    env->ThrowError("CMVSource: Couldn't open \'%s\'", filename);
  
  cmvStrm.seekg(0, std::ios::end);
  int cmvLen = static_cast<int>(cmvStrm.tellg());
  cmvStrm.seekg(0, std::ios::beg);
  
  std::vector<char> cmvBuf(cmvLen);
  cmvStrm.read(&cmvBuf[0], cmvLen);
  cmvStrm.close();

  const std::string DFdir = args[1].AsString("C:/Dwarf Fortress/");

  // Since I'm making use of Invoke a few times, I want any error messages seen
  // by users to indicate where they came from, so this little note gets added
  // to some of the exceptions seen later on; otherwise you'd get errors about
  // ImageSource, for example, even if you didn't use it in your script.
  const std::string errMine = "CMVSource internal error:\n";
  
  PClip tileset = NULL;

  if (args[2].IsClip()) {

    if ( !args[2].AsClip()->GetVideoInfo().IsRGB() )
      env->ThrowError("CMVSource: 'tileset' clips must be RGB!");

    tileset = args[2].AsClip();

  } else {

    std::ifstream initFileStrm( (DFdir + "/data/init/init.txt"),
                                std::ios::binary | std::ios::in );
    
    if (!initFileStrm)
      env->ThrowError("CMVSource: Invalid 'DFdir'!");
    
    std::string initBuf;
    while (initFileStrm.good())
      initBuf.push_back(static_cast<char>(initFileStrm.get()));

    initFileStrm.close();

    std::string fullfont;
    GetTokenValue("FULLFONT", initBuf) >> fullfont;
    std::string tilesDefault = DFdir + "/data/art/" + fullfont;

    // Dwarf Fortress does not currently support animated tilesets, and I don't
    // know if it ever will, so for now it's enough to grab a single frame and
    // pass it on to the CmvSource constructor. Otherwise I could do something in
    // GetFrame to get a new frame of the tileset for every frame of the CMV.
    AVSValue imgSrcArgs[3] = { args[2].AsString(tilesDefault.c_str()), 0, "RGB32" };
    const char* imgSrcArgNames[3] = { NULL, "end", "pixel_type" };

    try {
      tileset = env->Invoke("ImageSource", AVSValue(imgSrcArgs, 3), imgSrcArgNames).AsClip();

      // Even when requesting pixel_type="RGB32" with ImageSource/Reader, some
      // files come through as RGB24, so I run through a ConvertTo filter just
      // in case. I've only seen this with BMPs, but better safe than sorry; it
      // should be a no-op anyway if the data's already RGB32.
      tileset = env->Invoke("ConvertToRGB32", tileset).AsClip();
    } catch (AvisynthError& err) {
      std::string errOut = errMine + err.msg;
      env->ThrowError(errOut.c_str());
    }

  }

  // The only reliable way I could find to decide between pulling a color key
  // on magenta (for the original style tilesets) and using the existing alpha
  // channel data (for newer PNG tilesets) is to loop through all the pixels and
  // find out if there's any variation. If all the alpha components are 255,
  // there essentially is no transparency info, and a color key is called for.
  bool key = true;
  const unsigned char* shtp = tileset->GetFrame(0, env)->GetReadPtr();
  const int SHT_PITCH = tileset->GetFrame(0, env)->GetPitch();
  
  for (int h = 0; h < tileset->GetVideoInfo().height; ++h) {
    int alpha = 255;
    int shtLine = SHT_PITCH * h;
    for (int w = 0; w < tileset->GetVideoInfo().width; ++w) {
      int alphaOffset = shtLine + (w * 4) + 3;
      alpha = *(shtp + alphaOffset);
      if (alpha < 255) {
        key = false;
        break;
      }
    }
    if (alpha < 255)
      break;
  }

  if (key) {
    AVSValue keyArgs[2] = { tileset, 0xFF00FF };
    try {
      tileset = env->Invoke("ColorKeyMask", AVSValue(keyArgs, 2)).AsClip();
    } catch (AvisynthError& err) {
      std::string errOut = errMine + err.msg;
      env->ThrowError(errOut.c_str());
    }
  }

  const std::string colors = args[3].AsString((DFdir + "/data/init/colors.txt").c_str());

  // DF tilesets always have 16 columns and 16 rows of tiles, so tile size
  // computation is as easy as it gets.
  int tileW = tileset->GetVideoInfo().width / 16,
      tileH = tileset->GetVideoInfo().height / 16;

  return new CmvSource(cmvBuf, tileset, colors, tileW, tileH, env);

};



extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env)
{

  env->AddFunction("CMVSource", "s[DFdir]s[tileset]c[colors]s", Create_CmvSource, 0);
  env->AddFunction("CMVSource", "s[DFdir]s[tileset]s[colors]s", Create_CmvSource, 0);
  
  return "'CMVSource' - Loads Dwarf Fortress CMV and CCMV movies";

};